<?xml version="1.0"?>
<xsl:transform xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  version="1.0" xmlns:xs="http://www.w3.org/2001/XMLSchema">

  <xsl:output method="text"/>

  <xsl:strip-space elements="*"/>

  <xsl:template match="xs:attribute" mode="attributes">
    <xsl:text>&#xa;  </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:choose>
      <xsl:when test="xs:simpleType">
	<xsl:apply-templates select="xs:simpleType"/>
      </xsl:when>
      <xsl:when test="@type">
	<xsl:call-template name="simpleType">
	  <xsl:with-param name="type" select="@type"/>
	</xsl:call-template>
      </xsl:when>
    </xsl:choose>
    <xsl:choose>
      <xsl:when test="@use = 'required'">
	<xsl:text> #REQUIRED</xsl:text>
      </xsl:when>
      <xsl:when test="@default">
	<xsl:text> "</xsl:text>
	<xsl:value-of select="@default"/>
	<xsl:text>"</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text> #IMPLIED</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="xs:attribute" mode="attr-comments">
    <xsl:text>&#xa;</xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>: </xsl:text>
    <xsl:value-of select="xs:annotation/xs:documentation"/>
    <xsl:text>&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="xs:attribute"/>

  <xsl:template match="xs:choice">
    <xsl:text> (</xsl:text>
    <xsl:for-each select="*">
      <xsl:apply-templates select="."/>
      <xsl:if test="position() != last()">
	<xsl:text> |</xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text> )</xsl:text>
    <xsl:call-template name="occurrence"/>
  </xsl:template>

  <xsl:template match="xs:complexType">
    <xsl:choose>
      <xsl:when test="xs:all">
	<xsl:text> ANY</xsl:text>
      </xsl:when>
      <xsl:when test="xs:choice | xs:complexContent | xs:group |
        xs:sequence | xs:simpleContent">
	<xsl:apply-templates/>
      </xsl:when>
      <xsl:when test="@mixed='true'">
	<xsl:text> (#PCDATA)</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text> EMPTY</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="xs:documentation">
    <xsl:text>&#xa;&lt;!--&#xa;</xsl:text>
    <xsl:apply-templates/>
    <xsl:text>&#xa;-->&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="xs:element[@name]">
    <xsl:text>&#xa;&lt;!-- ELEMENT </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text> --></xsl:text>
    <xsl:apply-templates select="xs:annotation"/>
    <xs:text>&lt;!ELEMENT </xs:text>
    <xsl:value-of select="@name"/>
    <xsl:choose>
      <xsl:when test="xs:complexType">
	<xsl:apply-templates select="xs:complexType"/>
      </xsl:when>
      <xsl:when test="xs:simpleType">
	<xsl:text> (#PCDATA)</xsl:text>
      </xsl:when>
      <xsl:when test="@type">
	<xsl:text> (#PCDATA)</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text> EMPTY</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text> >&#xa;</xsl:text>
    <xsl:if test="descendant::xs:attribute">
      <xsl:text>&lt;!--</xsl:text>
      <xsl:apply-templates select="descendant::xs:attribute"
        mode="attr-comments"/>
      <xsl:text>-->&#xa;</xsl:text>
    </xsl:if>
    <xsl:text>&lt;!ATTLIST </xsl:text>
    <xsl:value-of select="@name"/>
    <xsl:text>&#xa;  xmlns CDATA #FIXED "</xsl:text>
    <xsl:value-of select="/xs:schema/@targetNamespace"/>
    <xsl:text>"</xsl:text>
    <xsl:if test="descendant::xs:attribute">
      <xsl:apply-templates select="descendant::xs:attribute"
        mode="attributes"/>
    </xsl:if>
    <xsl:text> >&#xa;</xsl:text>
  </xsl:template>

  <xsl:template match="xs:element[@ref]">
    <xsl:text> </xsl:text>
    <xsl:value-of select="@ref"/>
    <xsl:call-template name="occurrence"/>
  </xsl:template>

  <xsl:template match="xs:restriction">
    <xsl:choose>
      <xsl:when test="xs:enumeration">
	<xsl:text> ( </xsl:text>
	<xsl:for-each select="xs:enumeration">
	  <xsl:value-of select="@value"/>
	  <xsl:if test="position() != last()">
	    <xsl:text> | </xsl:text>
	  </xsl:if>
	</xsl:for-each>
	<xsl:text> )</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<xsl:call-template name="simpleType">
	  <xsl:with-param name="type" select="@base"/>
	</xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="xs:schema">
    <xsl:text>&lt;!-- Note that the original schema </xsl:text>
    <xsl:text>declarations may be stricter than&#xa;</xsl:text>
    <xsl:text>     represented here; please see the </xsl:text>
    <xsl:text>documentation for each element&#xa;</xsl:text>
    <xsl:text>     type. -->&#xa;</xsl:text>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="xs:sequence">
    <xsl:text> (</xsl:text>
    <xsl:for-each select="*">
      <xsl:apply-templates select="."/>
      <xsl:if test="position() != last()">
	<xsl:text>,</xsl:text>
      </xsl:if>
    </xsl:for-each>
    <xsl:text> )</xsl:text>
    <xsl:call-template name="occurrence"/>
  </xsl:template>

  <xsl:template name="occurrence">
    <xsl:choose>
      <xsl:when test="@minOccurs = 0 and @maxOccurs = 1">
	<xsl:text>?</xsl:text>
      </xsl:when>
      <xsl:when test="@minOccurs = 0">
	<xsl:text>*</xsl:text>
      </xsl:when>
      <xsl:when test="@maxOccurs = 1"/>
      <xsl:otherwise>
	<xsl:text>+</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <!-- only used by attributes -->
  <xsl:template name="simpleType">
    <xsl:param name="type"/>
    <xsl:choose>
      <xsl:when test="$type = 'xs:boolean'">
	<xsl:text> ( true | false )</xsl:text>
      </xsl:when>
      <xsl:otherwise>
	<xsl:text> CDATA</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:transform>
